% This program corresponds to Appendix 6.2.2 of the thesis.
% The effective rotational restraint contribution of a Timoshenko beam to the connected column is derived
clc, clear, close all

% Write the governing equations
syms m1 m2 m3 EI0 Rl tl Yj N kAG1 kAG2 kAG3 yi yj x
syms C1 C2 C3 C4 C5 C6 
y1(x) = C1 + C2*x + Yj/kAG1*x - Rl*tl/(2*EI0*m1)*x^2 - Yj/(6*EI0*m1)*x^3;
y2(x) = C3 + C4*x + Yj/kAG2*x - Rl*tl/(2*EI0*m2)*x^2 - Yj/(6*EI0*m2)*x^3;
y3(x) = C5 + C6*x + Yj/kAG3*x - Rl*tl/(2*EI0*m3)*x^2 - Yj/(6*EI0*m3)*x^3;
psi1(x) = C2 - Rl*tl/(EI0*m1)*x - Yj/(2*EI0*m1)*x^2;
psi2(x) = C4 - Rl*tl/(EI0*m2)*x - Yj/(2*EI0*m2)*x^2;
psi3(x) = C6 - Rl*tl/(EI0*m3)*x - Yj/(2*EI0*m3)*x^2;

% Substitute boundary conditions
syms L1 L2 L3 L tu Ru Tl Tu
eqn1 = subs(y1,x,0) == yi;
eqn2 = subs(y3,x,L) == yj;
eqn3 = subs(psi1,x,0) == Tl-tl;
eqn4 = subs(psi3,x,L) == Tu-tu;
eqn5 = tu*Ru + tl*Rl + Yj*L == 0;%-N*(yj-yi);
eqn6 = L1+L2+L3 == L;
eqn7 = subs(y1,x,L1) == subs(y2,x,L1);
eqn8 = subs(y2,x,L1+L2) == subs(y3,x,L1+L2);
eqn9 = subs(psi1,x,L1) == subs(psi2,x,L1);
eqn10 = subs(psi2,x,L1+L2) == subs(psi3,x,L1+L2);

% Solve for the applicable variables (isolated in the solution)
[A,B] = equationsToMatrix([eqn1,eqn2,eqn3,eqn4,eqn5,eqn6,eqn7,eqn8,eqn9,eqn10],[Yj,tu,tl,C1,C2,C3,C4,C5,C6,L3]);
X = linsolve(A,B)
ti_f = simplify(X(3));
tj_f = simplify(X(2));

% Invert the rotational stiffness matrix
[rotate_matrix,b] = equationsToMatrix([X(3);X(2)]==[tl; tu],[Tl;Tu;yi;yj]);
R_matrix = simplify([Rl 0; 0 Ru]*rotate_matrix)
syms ri rj taul tauu kAG0 u1 u2 u3 n0
R_matrix = simplify(subs(R_matrix,[Rl Ru],3*EI0/L*[ri/(1-ri)/taul rj/(1-rj)/tauu]))
R_matrix = simplify(subs(R_matrix,[kAG1 kAG2 kAG3],kAG0*[u1 u2 u3]))
R_matrix = simplify(subs(R_matrix,EI0,n0*kAG0*L^2))

%% Verifiy the hand-simplified solution to the above obtained solution
g1 = 3*n0/L*m1*m2*m3*(L1/u1+L2/u2+L3/u3);
g2 = 36*n0/L^2*m1*m2*m3*(L1^2/(m1*u1) + L2^2/(m2*u2) + L3^2/(m3*u3) +...
        L1*L3/(m1*u3) + L1*L3/(m3*u1) + L1*L2/(m1*u2) + L1*L2/(m2*u1) +...
        L2*L3/(m2*u3) + L2*L3/(m3*u2));
ci1 = 1*(L1^4*m2*m3 + L2^4*m1*m3 + L3^4*m1*m2)/L^4;
ci2 = 3*(L1*L2^3*m2*m3 + L1*L3^3*m2*m3 + L2*L3^3*m1*m3)/L^4;
ci3 = 4*(L2*L1^3*m2*m3 + L3*L1^3*m2*m3 + L3*L2^3*m1*m3)/L^4;
ci4 = 1*(L1*L2^3*m1*m3 + L1*L3^3*m1*m2 + L2*L3^3*m1*m2)/L^4;
ci5 = 6*(L1^2*L2^2*m2*m3 + L1^2*L3^2*m2*m3 + L2^2*L3^2*m1*m3)/L^4;
ci6 = 9*(L1*L2^2*L3*m2*m3 + L1*L2*L3^2*m2*m3)/L^4;
ci7 = 12*(L1^2*L2*L3*m2*m3)/L^4;
ci8 = 3*(L1*L2^2*L3*m1*m3 + L1*L2*L3^2*m1*m3)/L^4;
kN = ci1+ci2+ci3+ci4+ci5+ci6+ci7+ci8;
kN = kN + g1;
cj1 = 1*(L1^4*m2*m3 + L2^4*m1*m3 + L3^4*m1*m2)/L^4;
cj2 = 3*(L2*L1^3*m1*m3 + L3*L1^3*m1*m2 + L3*L2^3*m1*m2)/L^4;
cj3 = 4*(L1*L2^3*m1*m3 + L1*L3^3*m1*m2 + L2*L3^3*m1*m2)/L^4;
cj4 = 1*(L2*L1^3*m2*m3 + L3*L1^3*m2*m3 + L3*L2^3*m1*m3)/L^4;
cj5 = 6*(L1^2*L2^2*m1*m3 + L1^2*L3^2*m1*m2 + L2^2*L3^2*m1*m2)/L^4;
cj6 = 9*(L1^2*L2*L3*m1*m2 + L1*L2^2*L3*m1*m2)/L^4;
cj7 = 12*(L1*L2*L3^2*m1*m2)/L^4;
cj8 = 3*(L1^2*L2*L3*m1*m3 + L1*L2^2*L3*m1*m3)/L^4;
kF = cj1+cj2+cj3+cj4+cj5+cj6+cj7+cj8;
kF = kF + g1;
lii = 1/L^3*(L1^3*m2*m3 + L2^3*m1*m3 + L3^3*m1*m2  +  ...
             3*L1*L2^2*m1*m3 + 3*L1*L3^2*m1*m2 + 3*L2*L1^2*m1*m3 + ...
             3*L2*L3^2*m1*m2 + 3*L3*L1^2*m1*m2 + 3*L3*L2^2*m1*m2 + ...
             6*L1*L2*L3*m1*m2);
lii = lii + g1;
lij = 1/L^3*(L1^3*m2*m3 + L2^3*m1*m3 + L3^3*m1*m2  +  ...
             3*L1*L2^2*m1*m3 + 3*L1*L3^2*m1*m2 + 3*L2*L1^2*m2*m3 + ...
             3*L2*L3^2*m1*m2 + 3*L3*L1^2*m2*m3 + 3*L3*L2^2*m1*m3 + ...
             6*L1*L2*L3*m1*m3);
lij = lij - 2*g1;
gi =    tauu*m1*m2*m3*(1 - rj)  +  rj*lii;
gij = 2*tauu*m1*m2*m3*(1 - rj) + 3*rj*m1*m2*m3*(...
    L1^2/m1 + L2^2/m2 + L3^2/m3 + 2*L1*L3/m3 + 2*L2*L3/m3 + 2*L1*L2/m2);
lA = m1*m2*m3*taul*tauu;
lB = 4*tauu*(kN-taul*m1*m2*m3);
lC = 4*taul*(kF-tauu*m1*m2*m3);
lD = 4*(tauu*kN + taul*kF - tauu*taul*m1*m2*m3) - ...
      3/L^4*(L1^4*m2*m3/m1 + L2^4*m1*m3/m2 + L3^4*m1*m2/m3 + ...
            4*L1*L2^3*m3 + 4*L1*L3^3*m2 + 4*L2*L1^3*m3 + ...
            4*L2*L3^3*m1 + 4*L3*L1^3*m2 + 4*L3*L2^3*m1 + ...
            6*L1^2*L2^2*m3 + 6*L1^2*L3^2*m2 + 6*L2^2*L3^2*m1 + ...
            12*L1^2*L2*L3*m2 + 12*L1*L2^2*L3*m2 + 12*L1*L2*L3^2*m2);
lD = lD - g2;
R11_s = 6*EI0/L*(ri)/(4*lA  +  ri*lB  +  rj*lC  -  ri*rj*lD)*(2*gi);
R12_s = 6*EI0/L*(ri)/(4*lA  +  ri*lB  +  rj*lC  -  ri*rj*lD)*(lij*rj);
R13_s = 6*EI0/L^2 / (4*lA  +  ri*lB  +  rj*lC  -  ri*rj*lD)*(gij);

% R11 (first term in the solution)
zero_R11 = R11_s - R_matrix(1,1);
zero_R11 = subs(zero_R11,taul, 1/m1*(L1/L)^3  +  ...
               1/m2*((L1 + L2)^3/L^3  -  (L1/L)^3)  + ...
               1/m3*(1 - (L1 + L2)^3/L^3) );
zero_R11 = subs(zero_R11,tauu, 1/m3*(L3/L)^3  +  ...
               1/m2*((L3 + L2)^3/L^3  -  (L3/L)^3)  + ...
               1/m1*(1 - (L3 + L2)^3/L^3) );
zero_R11 = subs(zero_R11,n0, EI0/L^2/kAG0);
zero_R11 = subs(zero_R11,L,L1+L2+L3);
simplify(zero_R11) % should be zero

% R12 (second term in the solution)
zero_R12 = R12_s - R_matrix(1,2);
zero_R12 = subs(zero_R12,taul, 1/m1*(L1/L)^3  +  ...
               1/m2*((L1 + L2)^3/L^3  -  (L1/L)^3)  + ...
               1/m3*(1 - (L1 + L2)^3/L^3) );
zero_R12 = subs(zero_R12,tauu, 1/m3*(L3/L)^3  +  ...
               1/m2*((L3 + L2)^3/L^3  -  (L3/L)^3)  + ...
               1/m1*(1 - (L3 + L2)^3/L^3) );
zero_R12 = subs(zero_R12,n0, EI0/L^2/kAG0);
zero_R12 = subs(zero_R12,L,L1+L2+L3);
simplify(zero_R12) % should be zero

% R13 (third term in the solution)
zero_R13 = R12_s - R_matrix(1,2);
zero_R13 = subs(zero_R13,taul, 1/m1*(L1/L)^3  +  ...
               1/m2*((L1 + L2)^3/L^3  -  (L1/L)^3)  + ...
               1/m3*(1 - (L1 + L2)^3/L^3) );
zero_R13 = subs(zero_R13,tauu, 1/m3*(L3/L)^3  +  ...
               1/m2*((L3 + L2)^3/L^3  -  (L3/L)^3)  + ...
               1/m1*(1 - (L3 + L2)^3/L^3) );
zero_R13 = subs(zero_R13,n0, EI0/L^2/kAG0);
zero_R13 = subs(zero_R13,L,L1+L2+L3);
simplify(zero_R13) % should be zero
