% This program corresponds to Section 6.2.4 and Appendix A6.2.4 of the thesis.
% The effective rotational restraint contribution of a Timoshenko three-segment beam to the connected column is derived
% Note that the solution for a Euler-Bernoulli beam can be obtained by setting omega = 1 (or equivalently, kAG = Inf or eta = 0) 
clc, clear, close all

% Timoshenko system of governing DE's for three segment derivation
syms y1(x) y2(x) y3(x) psi1(x) psi2(x) psi3(x)
syms PI L1 L2 L3 L N Q
syms D D0 tu Ru tl Rl
syms C1 C2 C3 C4 C5 C6 
syms p1 p2 p3 w1 w2 w3 % let p1, p2, p3 be the non-adjusted phi factors
y1(x) = C1*cos(w1*p1/L*x) + C2*sin(w1*p1/L*x) + tl*Rl/N - D0/L*x - Q/N*x;
y2(x) = C3*cos(w2*p2/L*x) + C4*sin(w2*p2/L*x) + tl*Rl/N - D0/L*x - Q/N*x;
y3(x) = C5*cos(w3*p3/L*x) + C6*sin(w3*p3/L*x) + tl*Rl/N - D0/L*x - Q/N*x;
psi1(x) = 1/w1^2*(C2*w1*p1/L*cos(w1*p1/L*x) - C1*w1*p1/L*sin(w1*p1/L*x))-D0/L-Q/N;
psi2(x) = 1/w2^2*(C4*w2*p2/L*cos(w2*p2/L*x) - C3*w2*p2/L*sin(w2*p2/L*x))-D0/L-Q/N;
psi3(x) = 1/w3^2*(C6*w3*p3/L*cos(w3*p3/L*x) - C5*w3*p3/L*sin(w3*p3/L*x))-D0/L-Q/N; 

% Apply boundary conditions
eqn1 = y1(0) == 0;
eqn2 = y3(L) == D;
eqn3 = psi1(0) == tl;
eqn4 = psi3(L) == tu;
eqn5 = y1(L1) == y2(L1);
eqn6 = y2(L1+L2) == y3(L1+L2);
eqn7 = psi1(L1) == psi2(L1);
eqn8 = psi2(L1+L2) == psi3(L1+L2);
eqn9 = tu*Ru + tl*Rl == N*(D+D0) + Q*L;
eqn10 = L - L1 - L2 - L3 == 0;

% Solve the system of equations
[A,B] = equationsToMatrix(...
    [eqn1,eqn2,eqn3,eqn4,eqn5,eqn6,eqn7,eqn8,eqn9,eqn10],...
    [D,C1,C2,C3,C4,C5,C6,tu,tl,L3]);
X = linsolve(A,B)
Delta_sol = simplify(X(1))

% The above solution was simplified by hand and written as follows for verification
syms EI0 m1 m2 m3 kAG1 kAG2 kAG3
Delta_sol = subs(Delta_sol,[p1 p2 p3],L*(N/EI0)^0.5*[1/m1^0.5 1/m2^0.5 1/m3^0.5]);
Delta_sol = subs(Delta_sol,[w1 w2 w3],[(1+N/kAG1)^0.5 (1+N/kAG2)^0.5 (1+N/kAG3)^0.5]);
Delta_sol = subs(Delta_sol,[L],[L1+L2+L3]);
L = L1+L2+L3;
w1 = (1+N/kAG1)^0.5;
w2 = (1+N/kAG2)^0.5;
w3 = (1+N/kAG3)^0.5;
taul = 1/m1*(L1/L)^3 + 1/m2*(((L1+L2)/L)^3 - (L1/L)^3) + 1/m3*(1-((L1+L2)/L)^3);
tauu = 1/m3*(L3/L)^3 + 1/m2*(((L3+L2)/L)^3 - (L3/L)^3) + 1/m1*(1-((L3+L2)/L)^3);
rl = 1/(1+3*EI0/(L*Rl*taul));
ru = 1/(1+3*EI0/(L*Ru*tauu));
p0 = L*(N/EI0)^0.5;
p1 = p0/m1^0.5;
p2 = p0/m2^0.5;
p3 = p0/m3^0.5;
a1 = f(p1,p2,p3,w1,w2,w3,L1,L2,L3,L,0,2,0)...
    -f(p1,p2,p3,w1,w2,w3,L1,L2,L3,L,1,1,0)...
    -f(p1,p2,p3,w1,w2,w3,L1,L2,L3,L,0,1,1)...
    -f(p1,p2,p3,w1,w2,w3,L1,L2,L3,L,1,0,1);
a2 = f(p1,p2,p3,w1,w2,w3,L1,L2,L3,L,2,1,1)...
    +f(p1,p2,p3,w1,w2,w3,L1,L2,L3,L,1,2,1)...
    +f(p1,p2,p3,w1,w2,w3,L1,L2,L3,L,1,1,2)...
    -f(p1,p2,p3,w1,w2,w3,L1,L2,L3,L,2,0,2);
a3 = f(p1,p2,p3,w1,w2,w3,L1,L2,L3,L,2,1,0)...
    +f(p1,p2,p3,w1,w2,w3,L1,L2,L3,L,1,2,0)...
    +f(p1,p2,p3,w1,w2,w3,L1,L2,L3,L,2,0,1)...
    -f(p1,p2,p3,w1,w2,w3,L1,L2,L3,L,1,1,1);
a4 = f(p1,p2,p3,w1,w2,w3,L1,L2,L3,L,1,0,2)...
    +f(p1,p2,p3,w1,w2,w3,L1,L2,L3,L,0,1,2)...
    +f(p1,p2,p3,w1,w2,w3,L1,L2,L3,L,0,2,1)...
    -f(p1,p2,p3,w1,w2,w3,L1,L2,L3,L,1,1,1);
a5 = a3+a4 + 2*p1*p2*p3*w1*w2*w3;
NUM = p0^4*taul*tauu*(1-rl)*(1-ru)*a1 ...
        +9*rl*ru*a2 ...
        - 3*p0^2*(  rl*(1-ru)*tauu*a3 + ru*(1-rl)*taul*a4  );
DEN = 9*ru*rl*(a5-a2)...
    + 3*p0^2*(  rl*(1-ru)*tauu*(a3-a1) + ru*(1-rl)*taul*(a4-a1)  )...
    -p0^4*taul*tauu*(1-rl)*(1-ru)*a1;
beta = p0^2/12 * NUM/DEN;
S = 12*EI0/L^3*beta;
Delta_try = (N*D0/L + Q) / S;
zero = Delta_sol-Delta_try
%simplify(zero) % takes too long but the numerical result seems to work
zero_subs = subs(Delta_sol-Delta_try,[N EI0 L1 L2 L3 D0 Q Rl Ru m1 m2 m3 kAG1 kAG2 kAG3],...
    [2e6 2e11*1.27e-4 2.1 1.8 3.2 0.073 50000 4.02e7 3.16e7 0.8 0.7 0.6 0.44e-3*77e9 0.56e-3*77e9 0.84e-3*77e9])
zero_subs = double(zero_subs)

% Converts p1^a * p2^b * p3^c into the appropriate terms
% Make sure the input p1,p2,p3,w1,w2,w3,L1,L2,L3,L are symbolic
function [terms] = f(p1,p2,p3,w1,w2,w3,L1,L2,L3,L,a,b,c) 
    terms = p1^a*p2^b*p3^c;
    terms = terms*w1^(2-a)*w2^(2-b)*w3^(2-c);
    C1 = cos(w1*p1*L1/L); S1 = sin(w1*p1*L1/L);
    C2 = cos(w2*p2*L2/L); S2 = sin(w2*p2*L2/L);
    C3 = cos(w3*p3*L3/L); S3 = sin(w3*p3*L3/L);
    terms = terms*C1^mod(a,2)*C2^mod(b,2)*C3^mod(c,2);
    terms = terms*S1^mod(a+1,2)*S2^mod(b+1,2)*S3^mod(c+1,2);
end
