% Calculates the vertical stiffness, Sij, of a beam given:
%   E0 = ambient modulus, N/m2
%   Ib = moment of inertia of beam, m4
%   L1, L2, L3 = scalars, length of beam segment, m
%   m1, m2, m3 = scalars, degradation factors of segments
%   rL, rR = scalars, end fixity factors
%   tauL, tauR = scalars, modulus modification coefficients
function [Sij] = GetBeamStiffness(E0,Ib,L1,L2,L3,m1,m2,m3,rL,rR,tauL,tauR)

    % Verify input
    if (length(L1+L2+L3+m1+m2+m3+rL+rR+tauL+tauR) ~= 1)
        Sij = -100;
        return;
    end
    
    L = L1+L2+L3;
    OM = 3*rR*(1-rL)*tauL + 3*rL*(1-rR)*tauR;
    lb0 = 1/L^3*(L1^3*m2*m3/m1 + L2^3*m1*m3/m2 + L3^3*m1*m2/m3 + ...
                3*(L1^2*L2*m3 + L1^2*L3*m2 + L2^2*L1*m3 + L2^2*L3*m1 + L3^2*L1*m2 + L3^2*L2*m1) +...
                6*(L1*L2*L3*m2));
    lbF = 1/L^2*(L1^2*m2*m3 + L2^2*m1*m3 + L3^2*m1*m2 + ...
                2*(L1*L2*m1*m3+L1*L3*m1*m2+L2*L3*m1*m2));
    lbN = 1/L^2*(L1^2*m2*m3 + L2^2*m1*m3 + L3^2*m1*m2 + ...
                2*(L1*L2*m2*m3+L1*L3*m2*m3+L2*L3*m1*m3));

    k0 = 1/L^4*(L1^3*(2*L-L1)*m2*m3/m1 + L2^3*(2*L-L2)*m1*m3/m2 + L3^3*(2*L-L3)*m1*m2/m3 + ...
                2*(L1*L2^3*m3 + L1*L3^3*m2 + L2*L1^3*m3 + L2*L3^3*m1 + L3*L1^3*m2 + L3*L2^3*m1)+...
                6*(L1^2*L2^2*m3 + L1^2*L3^2*m2 + L2^2*L3^2*m1)+...
                6*(L1^2*L2*L3*(m2+m3) + L2^2*L1*L3*(m1+m3) + L3^2*L1*L2*(m1+m2)));
    n0 = tauL*tauR*(rL-1)*(rR-1)*m1*m2*m3;
    n1 = 3*rR*(1-rL)*tauL + 3*rL*(1-rR)*tauR + 9*rL*rR/L*(L1/m1 + L2/m2 + L3/m3);
    n2 = 3*rL*rR*lb0 + 3*rR*(1-rL)*tauL*lbF + 3*rL*(1-rR)*tauR*lbN;

    lb_NF = 1/L^3* ( L1^3*m2*m3 + L2^3*m1*m3 + L3^3*m1*m2+...
              3*L1*L2^2*m1*m3 + 3*L1*L3^2*m1*m2+3*L2*L1^2*m2*m3+...
              3*L3*L2^2*m1*m3 + 3*L3*L1^2*m3*m2+3*L2*L3^2*m1*m2+...
              6*L1*L2*L3*m1*m3 );

    Sij =  12*E0*Ib/L^3* (n1*m1*m2*m3)/(12*n0+6*n2-2*lb_NF*OM-9*rL*rR*k0);
    
    % Special case - zero stiffness if beam segment has completely degraded
    if (m1<=0 || m2<=0 || m3<=0)
        Sij = 0;
        return;
    end
end