% Calculates the critical sway load, Psw
%   Ec = (n+1) vector of column moduli
%   Ic = (n+1) vector of column inertia
%   Lc = (n+1) vector of column lengths
%   ru = (n+1) vector of upper end fixity
%   rl = (n+1) vector of lower end fixity
%   Kb = scalar, total semi-bracing stiffness
function Psw = Get_Psw(Ec,Ic,Lc,ru,rl,Kb,lambda)
    % Verify input
    n = length(Ec)-1;
    if (length(Ic) ~= n+1 || length(Lc)~=n+1 || length(Ec)~=n+1 ||...
            length(ru)~=n+1|| length(rl)~=n+1 || length(Kb)~=1)
        Psw = -100;
        return;
    end
    
    % Initial guess
    K = Get_K(ru,rl);
    Pb = pi^2*Ec.*Ic./(K.*Lc).^2;
    Pmax = min(Pb./lambda);
    P_val = Pmax*0.5;
    
    max_iter = 100;
    tol = 1e-3;
    step = 1;
    for num_iter = 1:max_iter
        
        % Get S
        P = P_val*lambda;
        phi = Get_phi(P,Ec,Ic,Lc);
        beta = Get_beta(phi,ru,rl);
        [sumS,~] = Get_S_2001(P,Ec,Ic,Lc,beta);
        SS = sumS + Kb;
        
        % Get dS
        P_step = (P_val + step)*lambda;
        phi = Get_phi(P_step,Ec,Ic,Lc);
        beta = Get_beta(phi,ru,rl);
        [sumS_step,~] = Get_S_2001(P_step,Ec,Ic,Lc,beta);
        SS_step = sumS_step + Kb;
        dS = SS_step - SS;
        dSdP = dS/step;
        
        % Process and iterate
        P_next = P_val - SS/dSdP;
        if abs(P_next - P_val)<tol
            Psw = P_next;
            break;
        end
        if num_iter == max_iter
            Psw = -101;
            break;
        end
        P_val = P_next;
    end
end